@@ -2,7 +2,7 @@
 #include <math.h>
 #include <string.h>
 #include "canvas_demo.h"
-#include <libgnomecanvas/gnome-canvas-path-def.h>
+#include "gnome-canvas-path-def.h"
 #include <gdk/gdkkeysyms.h>
 
 static void
@@ -733,10 +733,8 @@ key_press (GnomeCanvas *canvas, GdkEventKey *event, gpointer data)
 		gnome_canvas_scroll_to (canvas, x - 10, y);
 	else if (event->keyval == GDK_Right)
 		gnome_canvas_scroll_to (canvas, x + 10, y);
-	else
-		return FALSE;
 
-	return TRUE;
+	return FALSE;
 }
 
 GtkWidget *
@@ -773,8 +771,6 @@ create_canvas_primitives (gint aa)
 	else
 	  canvas = gnome_canvas_new ();
 
-	gnome_canvas_set_center_scroll_region (GNOME_CANVAS (canvas), FALSE);
-
 	/* Setup canvas items */
 
 	root = gnome_canvas_root (GNOME_CANVAS (canvas));
@@ -789,8 +785,6 @@ create_canvas_primitives (gint aa)
 	setup_curves (root);
 	setup_widgets (root);
 
-/* FIXME: we should have a 'rotation' spinbutton too - and fix the acute
-   bugs with that ... */
 #if 0
 	{
 		double affine[6];
