@@ -111,27 +111,24 @@ static void   gnome_canvas_line_render      (GnomeCanvasItem *item, GnomeCanvasB
 static GnomeCanvasItemClass *parent_class;
 
 
-GType
+GtkType
 gnome_canvas_line_get_type (void)
 {
-	static GType line_type;
+	static GtkType line_type = 0;
 
 	if (!line_type) {
-		static const GTypeInfo object_info = {
-			sizeof (GnomeCanvasLineClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) gnome_canvas_line_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,			/* class_data */
+		GtkTypeInfo line_info = {
+			"GnomeCanvasLine",
 			sizeof (GnomeCanvasLine),
-			0,			/* n_preallocs */
-			(GInstanceInitFunc) gnome_canvas_line_init,
-			NULL			/* value_table */
+			sizeof (GnomeCanvasLineClass),
+			(GtkClassInitFunc) gnome_canvas_line_class_init,
+			(GtkObjectInitFunc) gnome_canvas_line_init,
+			NULL, /* reserved_1 */
+			NULL, /* reserved_2 */
+			(GtkClassInitFunc) NULL
 		};
 
-		line_type = g_type_register_static (GNOME_TYPE_CANVAS_ITEM, "GnomeCanvasLine",
-						    &object_info, 0);
+		line_type = gtk_type_unique (gnome_canvas_item_get_type (), &line_info);
 	}
 
 	return line_type;
@@ -148,7 +145,7 @@ gnome_canvas_line_class_init (GnomeCanvasLineClass *class)
 	object_class = (GtkObjectClass *) class;
 	item_class = (GnomeCanvasItemClass *) class;
 
-	parent_class = g_type_class_peek_parent (class);
+	parent_class = gtk_type_class (gnome_canvas_item_get_type ());
 
 	gobject_class->set_property = gnome_canvas_line_set_property;
 	gobject_class->get_property = gnome_canvas_line_get_property;
@@ -339,11 +336,6 @@ get_bounds (GnomeCanvasLine *line, double *bx1, double *by1, double *bx2, double
 	double width;
 	int i;
 
-	if (!line->coords) {
-	    *bx1 = *by1 = *bx2 = *by2 = 0.0;
-	    return;
-	}
-	
 	/* Find bounding box of line's points */
 
 	x1 = x2 = line->coords[0];
@@ -641,6 +633,16 @@ set_stipple (GnomeCanvasLine *line, GdkBitmap *stipple, int reconfigure)
 	}
 }
 
+#ifdef OLD_XFORM
+/* Convenience functions to recalculate the arrows and bounds of the line */
+static void
+reconfigure_arrows_and_bounds (GnomeCanvasLine *line)
+{
+	reconfigure_arrows (line);
+	recalc_bounds (line);
+}
+#endif
+
 static void
 gnome_canvas_line_set_property (GObject              *object,
 				guint                 param_id,
@@ -696,7 +698,12 @@ gnome_canvas_line_set_property (GObject              *object,
 		/* Since the line's points have changed, we need to re-generate arrowheads in
 		 * addition to recalculating the bounds.
 		 */
+
+#ifdef OLD_XFORM
+		reconfigure_arrows_and_bounds (line);
+#else
 		gnome_canvas_item_request_update (item);
+#endif
 		break;
 
 	case PROP_FILL_COLOR:
@@ -742,40 +749,68 @@ gnome_canvas_line_set_property (GObject              *object,
 		line->width = g_value_get_uint (value);
 		line->width_pixels = TRUE;
 		set_line_gc_width (line);
+#ifdef OLD_XFORM
+		reconfigure_arrows_and_bounds (line);
+#else
 		gnome_canvas_item_request_update (item);
+#endif
 		break;
 
 	case PROP_WIDTH_UNITS:
 		line->width = fabs (g_value_get_double (value));
 		line->width_pixels = FALSE;
 		set_line_gc_width (line);
+#ifdef OLD_XFORM
+		reconfigure_arrows_and_bounds (line);
+#else
 		gnome_canvas_item_request_update (item);
+#endif
 		break;
 
 	case PROP_CAP_STYLE:
 		line->cap = g_value_get_enum (value);
+#ifdef OLD_XFORM
+		reconfigure_arrows_and_bounds (line);
+#else
 		gnome_canvas_item_request_update (item);
+#endif
 		break;
 
 	case PROP_JOIN_STYLE:
 		line->join = g_value_get_enum (value);
+#ifdef OLD_XFORM
+		reconfigure_arrows_and_bounds (line);
+#else
 		gnome_canvas_item_request_update (item);
+#endif
 		break;
 
 	case PROP_LINE_STYLE:
 		line->line_style = g_value_get_enum (value);
 		set_line_gc_width (line);
+#ifdef OLD_XFORM
+		reconfigure_arrows_and_bounds (line);
+#else
 		gnome_canvas_item_request_update (item);
+#endif
 		break;
 
 	case PROP_FIRST_ARROWHEAD:
 		line->first_arrow = g_value_get_boolean (value);
+#ifdef OLD_XFORM
+		reconfigure_arrows_and_bounds (line);
+#else
 		gnome_canvas_item_request_update (item);
+#endif
 		break;
 
 	case PROP_LAST_ARROWHEAD:
 		line->last_arrow = g_value_get_boolean (value);
+#ifdef OLD_XFORM
+		reconfigure_arrows_and_bounds (line);
+#else
 		gnome_canvas_item_request_update (item);
+#endif
 		break;
 
 	case PROP_SMOOTH:
@@ -788,17 +823,29 @@ gnome_canvas_line_set_property (GObject              *object,
 
 	case PROP_ARROW_SHAPE_A:
 		line->shape_a = fabs (g_value_get_double (value));
+#ifdef OLD_XFORM
+		reconfigure_arrows_and_bounds (line);
+#else
 		gnome_canvas_item_request_update (item);
+#endif
 		break;
 
 	case PROP_ARROW_SHAPE_B:
 		line->shape_b = fabs (g_value_get_double (value));
+#ifdef OLD_XFORM
+		reconfigure_arrows_and_bounds (line);
+#else
 		gnome_canvas_item_request_update (item);
+#endif
 		break;
 
 	case PROP_ARROW_SHAPE_C:
 		line->shape_c = fabs (g_value_get_double (value));
+#ifdef OLD_XFORM
+		reconfigure_arrows_and_bounds (line);
+#else
 		gnome_canvas_item_request_update (item);
+#endif
 		break;
 
 	default:
@@ -817,57 +864,9 @@ gnome_canvas_line_set_property (GObject              *object,
 			set_line_gc_foreground (line);
 
 		gnome_canvas_item_request_redraw_svp (item, line->fill_svp);
-
-		if (line->first_svp) 
-			gnome_canvas_item_request_redraw_svp (item, line->first_svp);
-
-		if (line->last_svp) 
-			gnome_canvas_item_request_redraw_svp (item, line->last_svp);
-
 	}
 }
 
-/* Returns a copy of the line's points without the endpoint adjustments for
- * arrowheads.
- */
-static GnomeCanvasPoints *
-get_points (GnomeCanvasLine *line)
-{
-	GnomeCanvasPoints *points;
-	int start_ofs, end_ofs;
-
-	if (line->num_points == 0)
-		return NULL;
-
-	start_ofs = end_ofs = 0;
-
-	points = gnome_canvas_points_new (line->num_points);
-
-	/* Invariant:  if first_coords or last_coords exist, then the line's
-	 * endpoints have been adjusted.
-	 */
-
-	if (line->first_coords) {
-		start_ofs = 1;
-
-		points->coords[0] = line->first_coords[0];
-		points->coords[1] = line->first_coords[1];
-	}
-
-	if (line->last_coords) {
-		end_ofs = 1;
-
-		points->coords[2 * (line->num_points - 1)] = line->last_coords[0];
-		points->coords[2 * (line->num_points - 1) + 1] = line->last_coords[1];
-	}
-
-	memcpy (points->coords + 2 * start_ofs,
-		line->coords + 2 * start_ofs,
-		2 * (line->num_points - (start_ofs + end_ofs)) * sizeof (double));
-
-	return points;
-}
-
 static void
 gnome_canvas_line_get_property (GObject              *object,
 				guint                 param_id,
@@ -875,6 +874,8 @@ gnome_canvas_line_get_property (GObject              *object,
 				GParamSpec           *pspec)
 {
 	GnomeCanvasLine *line;
+	GnomeCanvasPoints *points;
+	GdkColor *color;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GNOME_IS_CANVAS_LINE (object));
@@ -883,24 +884,22 @@ gnome_canvas_line_get_property (GObject              *object,
 
 	switch (param_id) {
 	case PROP_POINTS:
-		g_value_set_boxed (value, get_points (line));
-		break;
-
-	case PROP_FILL_COLOR:
-		g_value_set_string_take_ownership (value,
-						   g_strdup_printf ("#%02x%02x%02x",
-								    line->fill_rgba >> 24,
-								    (line->fill_rgba >> 16) & 0xff,
-								    (line->fill_rgba >> 8) & 0xff));
+		if (line->num_points != 0) {
+			points = gnome_canvas_points_new (line->num_points);
+			memcpy (points->coords, line->coords, 2 * line->num_points * sizeof (double));
+			g_value_set_boxed (value, points);
+		} else
+			g_value_set_boxed (value, NULL);
 		break;
 
 	case PROP_FILL_COLOR_GDK: {
-		GnomeCanvas *canvas = GNOME_CANVAS_ITEM (line)->canvas;
-		GdkColormap *colormap = gtk_widget_get_colormap (GTK_WIDGET (canvas));
-		GdkColor color;
+		GdkColormap *colormap;
 
-		gdk_colormap_query_color (colormap, line->fill_pixel, &color);
-		g_value_set_boxed (value, &color);
+		color = g_new (GdkColor, 1);
+		color->pixel = line->fill_pixel;
+		colormap = gtk_widget_get_colormap (GTK_WIDGET (line->item.canvas));
+		gdk_rgb_find_color (colormap, color);
+		g_value_set_boxed (value, color);
 		break;
 	}
 
@@ -909,17 +908,9 @@ gnome_canvas_line_get_property (GObject              *object,
 		break;
 
 	case PROP_FILL_STIPPLE:
-		g_value_set_object (value, line->stipple);
+		g_value_set_boxed (value, line->stipple);
 		break;
 
-	case PROP_WIDTH_PIXELS:
-		g_value_set_uint (value, line->width);
-		break;
-		
-	case PROP_WIDTH_UNITS:
-		g_value_set_double (value, line->width);
-		break;
-		
 	case PROP_CAP_STYLE:
 		g_value_set_enum (value, line->cap);
 		break;
@@ -1094,6 +1085,10 @@ gnome_canvas_line_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_pa
 		get_bounds_canvas (line, &x1, &y1, &x2, &y2, affine);
 		gnome_canvas_update_bbox (item, x1, y1, x2, y2);
 	}
+
+#ifdef OLD_XFORM
+	reconfigure_arrows_and_bounds (line);
+#endif
 }
 
 static void
@@ -1129,7 +1124,7 @@ gnome_canvas_line_unrealize (GnomeCanvasItem *item)
 
 static void
 item_to_canvas (GnomeCanvas *canvas, double *item_coords, GdkPoint *canvas_coords, int num_points,
-		int *num_drawn_points, double i2c[6], int x, int y)
+		int *num_drawn_points, double i2c[6])
 {
 	int i;
 	int old_cx, old_cy;
@@ -1151,8 +1146,8 @@ item_to_canvas (GnomeCanvas *canvas, double *item_coords, GdkPoint *canvas_coord
 	art_affine_point (&pc, &pi, i2c);
 	cx = floor (pc.x + 0.5);
 	cy = floor (pc.y + 0.5);
-	canvas_coords->x = cx - x;
-	canvas_coords->y = cy - y;
+	canvas_coords->x = cx;
+	canvas_coords->y = cy;
 	canvas_coords++;
 	old_cx = cx;
 	old_cy = cy;
@@ -1165,8 +1160,8 @@ item_to_canvas (GnomeCanvas *canvas, double *item_coords, GdkPoint *canvas_coord
 		cx = floor (pc.x + 0.5);
 		cy = floor (pc.y + 0.5);
 		if (old_cx != cx || old_cy != cy) {
-			canvas_coords->x = cx - x;
-			canvas_coords->y = cy - y;
+			canvas_coords->x = cx;
+			canvas_coords->y = cy;
 			old_cx = cx;
 			old_cy = cy;
 			canvas_coords++;
@@ -1197,11 +1192,13 @@ gnome_canvas_line_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	else
 		points = g_new (GdkPoint, line->num_points);
 
-
 	gnome_canvas_item_i2c_affine (item, i2c);
 
+	i2c[4] -= x;
+	i2c[5] -= y;
+
 	item_to_canvas (item->canvas, line->coords, points, line->num_points,
-			&actual_num_points_drawn, i2c, x, y);
+			&actual_num_points_drawn, i2c);
 
 	if (line->stipple)
 		gnome_canvas_set_stipple_origin (item->canvas, line->gc);
@@ -1217,13 +1214,13 @@ gnome_canvas_line_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 
 	if (line->first_arrow) {
 		item_to_canvas (item->canvas, line->first_coords, points, NUM_ARROW_POINTS,
-				&actual_num_points_drawn, i2c, x, y);
+				&actual_num_points_drawn, i2c);
 		gdk_draw_polygon (drawable, line->gc, TRUE, points, actual_num_points_drawn );
 	}
 
 	if (line->last_arrow) {
 		item_to_canvas (item->canvas, line->last_coords, points, NUM_ARROW_POINTS,
-				&actual_num_points_drawn, i2c, x, y);
+				&actual_num_points_drawn, i2c);
 		gdk_draw_polygon (drawable, line->gc, TRUE, points, actual_num_points_drawn );
 	}
 }
