@@ -29,6 +29,8 @@
 #include <libart_lgpl/art_rgb_rgba_affine.h>
 #include "gnome-canvas-pixbuf.h"
 
+
+
 /* Private part of the GnomeCanvasPixbuf structure */
 typedef struct {
 	/* Our gdk-pixbuf */
@@ -51,7 +53,9 @@ typedef struct {
 	guint width_in_pixels : 1;
 	guint height_set : 1;
 	guint height_in_pixels : 1;
+	guint x_set : 1;
 	guint x_in_pixels : 1;
+	guint y_set : 1;
 	guint y_in_pixels : 1;
 	
 	/* Whether the pixbuf has changed */
@@ -64,6 +68,8 @@ typedef struct {
 	GtkAnchorType anchor;
 } PixbufPrivate;
 
+
+
 /* Object argument IDs */
 enum {
 	PROP_0,
@@ -75,8 +81,10 @@ enum {
 	PROP_HEIGHT_SET,
 	PROP_HEIGHT_IN_PIXELS,
 	PROP_X,
+	PROP_X_SET,
 	PROP_X_IN_PIXELS,
 	PROP_Y,
+	PROP_Y_SET,
 	PROP_Y_IN_PIXELS,
 	PROP_ANCHOR
 };
@@ -119,27 +127,25 @@ static GnomeCanvasItemClass *parent_class;
 GtkType
 gnome_canvas_pixbuf_get_type (void)
 {
-	static GType pixbuf_type;
+	static GtkType canvas_pixbuf_type = 0;
 
-	if (!pixbuf_type) {
-		static const GTypeInfo object_info = {
-			sizeof (GnomeCanvasPixbufClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) gnome_canvas_pixbuf_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,			/* class_data */
+	if (!canvas_pixbuf_type) {
+		static const GtkTypeInfo canvas_pixbuf_info = {
+			"GnomeCanvasPixbuf",
 			sizeof (GnomeCanvasPixbuf),
-			0,			/* n_preallocs */
-			(GInstanceInitFunc) gnome_canvas_pixbuf_init,
-			NULL			/* value_table */
+			sizeof (GnomeCanvasPixbufClass),
+			(GtkClassInitFunc) gnome_canvas_pixbuf_class_init,
+			(GtkObjectInitFunc) gnome_canvas_pixbuf_init,
+			NULL, /* reserved_1 */
+			NULL, /* reserved_2 */
+			(GtkClassInitFunc) NULL
 		};
 
-		pixbuf_type = g_type_register_static (GNOME_TYPE_CANVAS_ITEM, "GnomeCanvasPixbuf",
-						     &object_info, 0);
+		canvas_pixbuf_type = gtk_type_unique (gnome_canvas_item_get_type (),
+						      &canvas_pixbuf_info);
 	}
 
-	return pixbuf_type;
+	return canvas_pixbuf_type;
 }
 
 /* Class initialization function for the pixbuf canvas item */
@@ -154,7 +160,7 @@ gnome_canvas_pixbuf_class_init (GnomeCanvasPixbufClass *class)
 	object_class = (GtkObjectClass *) class;
 	item_class = (GnomeCanvasItemClass *) class;
 
-	parent_class = g_type_class_peek_parent (class);
+	parent_class = gtk_type_class (gnome_canvas_item_get_type ());
 
 	gobject_class->set_property = gnome_canvas_pixbuf_set_property;
 	gobject_class->get_property = gnome_canvas_pixbuf_get_property;
@@ -207,6 +213,12 @@ gnome_canvas_pixbuf_class_init (GnomeCanvasPixbufClass *class)
                  g_param_spec_double ("x", NULL, NULL,
 				      -G_MAXDOUBLE, G_MAXDOUBLE, 0,
                                     (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+        g_object_class_install_property
+                (gobject_class,
+                 PROP_X_SET,
+                 g_param_spec_boolean ("x_set", NULL, NULL,
+				       FALSE,
+				       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property
                 (gobject_class,
                  PROP_X_IN_PIXELS,
@@ -219,6 +231,12 @@ gnome_canvas_pixbuf_class_init (GnomeCanvasPixbufClass *class)
                  g_param_spec_double ("y", NULL, NULL,
 				      -G_MAXDOUBLE, G_MAXDOUBLE, 0,
 				      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+        g_object_class_install_property
+                (gobject_class,
+                 PROP_Y_SET,
+                 g_param_spec_boolean ("y_set", NULL, NULL,
+				       FALSE,
+				       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property
                 (gobject_class,
                  PROP_Y_IN_PIXELS,
@@ -255,7 +273,6 @@ gnome_canvas_pixbuf_init (GnomeCanvasPixbuf *gcp)
 	priv->height = 0.0;
 	priv->x = 0.0;
 	priv->y = 0.0;
-	priv->anchor = GTK_ANCHOR_NW;
 }
 
 /* Destroy handler for the pixbuf canvas item */
@@ -386,6 +403,12 @@ gnome_canvas_pixbuf_set_property (GObject            *object,
 		gnome_canvas_item_request_update (item);
 		break;
 
+	case PROP_X_SET:
+		priv->x_set = g_value_get_boolean (value);
+		priv->need_xform_update = TRUE;
+		gnome_canvas_item_request_update (item);
+		break;
+
 	case PROP_X_IN_PIXELS:
 		priv->x_in_pixels = g_value_get_boolean (value);
 		priv->need_xform_update = TRUE;
@@ -398,6 +421,12 @@ gnome_canvas_pixbuf_set_property (GObject            *object,
 		gnome_canvas_item_request_update (item);
 		break;
 
+	case PROP_Y_SET:
+		priv->y_set = g_value_get_boolean (value);
+		priv->need_xform_update = TRUE;
+		gnome_canvas_item_request_update (item);
+		break;
+
 	case PROP_Y_IN_PIXELS:
 		priv->y_in_pixels = g_value_get_boolean (value);
 		priv->need_xform_update = TRUE;
@@ -465,6 +494,10 @@ gnome_canvas_pixbuf_get_property (GObject            *object,
 		g_value_set_double (value, priv->x);
 		break;
 
+	case PROP_X_SET:
+		g_value_set_boolean (value, priv->x_set);
+		break;
+
 	case PROP_X_IN_PIXELS:
 		g_value_set_boolean (value, priv->x_in_pixels);
 		break;
@@ -473,6 +506,10 @@ gnome_canvas_pixbuf_get_property (GObject            *object,
 		g_value_set_double (value, priv->y);
 		break;
 
+	case PROP_Y_SET:
+		g_value_set_boolean (value, priv->y_set);
+		break;
+
 	case PROP_Y_IN_PIXELS:
 		g_value_set_boolean (value, priv->y_in_pixels);
 		break;
@@ -556,8 +593,15 @@ compute_viewport_affine (GnomeCanvasPixbuf *gcp, double *viewport_affine, double
 	else
 		h = gdk_pixbuf_get_height (priv->pixbuf);
 
-	x = priv->x;
-	y = priv->y;
+	if (priv->x_set)
+		x = priv->x;
+	else
+		x = 0.0;
+
+	if (priv->y_set)
+		y = priv->y;
+	else
+		y = 0.0;
 
 	/* Convert i_len and j_len into scaling factors */
 
@@ -595,7 +639,7 @@ compute_viewport_affine (GnomeCanvasPixbuf *gcp, double *viewport_affine, double
 	case GTK_ANCHOR_NW:
 	case GTK_ANCHOR_W:
 	case GTK_ANCHOR_SW:
-		ti_len *= x;
+		ti_len += x;
 		break;
 
 	case GTK_ANCHOR_N:
@@ -658,18 +702,12 @@ compute_viewport_affine (GnomeCanvasPixbuf *gcp, double *viewport_affine, double
  * by some other amount.
  */
 static void
-compute_render_affine (GnomeCanvasPixbuf *gcp, double *ra, double *i2c)
+compute_render_affine (GnomeCanvasPixbuf *gcp, double *render_affine, double *i2c)
 {
-	double va[6];
+	double viewport_affine[6];
 
-	compute_viewport_affine (gcp, va, i2c);
-#ifdef GNOME_CANVAS_PIXBUF_VERBOSE
-	g_print ("va %g %g %g %g %g %g\n", va[0], va[1], va[2], va[3], va[4], va[5]);
-#endif
-	art_affine_multiply (ra, va, i2c);
-#ifdef GNOME_CANVAS_PIXBUF_VERBOSE
-	g_print ("ra %g %g %g %g %g %g\n", ra[0], ra[1], ra[2], ra[3], ra[4], ra[5]);
-#endif
+	compute_viewport_affine (gcp, viewport_affine, i2c);
+	art_affine_multiply (render_affine, viewport_affine, i2c);
 }
 
 /* Recomputes the bounding box of a pixbuf canvas item.  The horizontal and
@@ -677,11 +715,11 @@ compute_render_affine (GnomeCanvasPixbuf *gcp, double *ra, double *i2c)
  * have to compute the components individually for each dimension.
  */
 static void
-recompute_bounding_box (GnomeCanvasPixbuf *gcp, gdouble *i2c)
+recompute_bounding_box (GnomeCanvasPixbuf *gcp)
 {
 	GnomeCanvasItem *item;
 	PixbufPrivate *priv;
-	double ra[6];
+	double i2c[6], render_affine[6];
 	ArtDRect rect;
 
 	item = GNOME_CANVAS_ITEM (gcp);
@@ -698,18 +736,9 @@ recompute_bounding_box (GnomeCanvasPixbuf *gcp, gdouble *i2c)
 	rect.y0 = 0.0;
 	rect.y1 = gdk_pixbuf_get_height (priv->pixbuf);
 
-#ifdef GNOME_CANVAS_PIXBUF_VERBOSE
-	g_print ("i2c %g %g %g %g %g %g\n", i2c[0], i2c[1], i2c[2], i2c[3], i2c[4], i2c[5]);
-#endif
 	gnome_canvas_item_i2c_affine (item, i2c);
-#ifdef GNOME_CANVAS_PIXBUF_VERBOSE
-	g_print ("i2c %g %g %g %g %g %g\n", i2c[0], i2c[1], i2c[2], i2c[3], i2c[4], i2c[5]);
-#endif
-	compute_render_affine (gcp, ra, i2c);
-#ifdef GNOME_CANVAS_PIXBUF_VERBOSE
-	g_print ("ra %g %g %g %g %g %g\n", ra[0], ra[1], ra[2], ra[3], ra[4], ra[5]);
-#endif
-	art_drect_affine_transform (&rect, &rect, ra);
+	compute_render_affine (gcp, render_affine, i2c);
+	art_drect_affine_transform (&rect, &rect, render_affine);
 
 	item->x1 = floor (rect.x0);
 	item->y1 = floor (rect.y0);
@@ -738,9 +767,8 @@ gnome_canvas_pixbuf_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_
 	     && !(GTK_OBJECT_FLAGS (item) & GNOME_CANVAS_ITEM_VISIBLE))
 	    || (flags & GNOME_CANVAS_UPDATE_AFFINE)
 	    || priv->need_pixbuf_update
-	    || priv->need_xform_update) {
+	    || priv->need_xform_update)
 		gnome_canvas_request_redraw (item->canvas, item->x1, item->y1, item->x2, item->y2);
-	}
 
 	/* If we need a pixbuf update, or if the item changed visibility to
 	 * shown, recompute the bounding box.
@@ -750,10 +778,7 @@ gnome_canvas_pixbuf_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_
 	    || ((flags & GNOME_CANVAS_UPDATE_VISIBILITY)
 		&& (GTK_OBJECT_FLAGS (gcp) & GNOME_CANVAS_ITEM_VISIBLE))
 	    || (flags & GNOME_CANVAS_UPDATE_AFFINE)) {
-		recompute_bounding_box (gcp, affine);
-#ifdef GNOME_CANVAS_PIXBUF_VERBOSE
-		g_print ("BBox is %g %g %g %g\n", item->x1, item->y1, item->x2, item->y2);
-#endif
+		recompute_bounding_box (gcp);
 		gnome_canvas_request_redraw (item->canvas, item->x1, item->y1, item->x2, item->y2);
 		priv->need_pixbuf_update = FALSE;
 		priv->need_xform_update = FALSE;
@@ -875,8 +900,8 @@ gnome_canvas_pixbuf_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 					     0, 0,
 					     d_rect.x0 - x, d_rect.y0 - y,
 					     w, h,
-					     GDK_PIXBUF_ALPHA_FULL,
-					     0,
+					     GDK_PIXBUF_ALPHA_BILEVEL,
+					     128,
 					     GDK_RGB_DITHER_MAX,
 					     d_rect.x0, d_rect.y0);
 
@@ -902,52 +927,7 @@ gnome_canvas_pixbuf_render (GnomeCanvasItem *item, GnomeCanvasBuf *buf)
 	compute_render_affine (gcp, render_affine, i2c);
         gnome_canvas_buf_ensure_buf (buf);
 
-
-	if (art_affine_rectilinear (render_affine))
-	  {
-	    GdkPixbuf *dest_pixbuf;
-	    int x0, y0, x1, y1;
-
-	    dest_pixbuf = gdk_pixbuf_new_from_data (buf->buf,
-						    GDK_COLORSPACE_RGB,
-						    FALSE,
-						    8,
-						    buf->rect.x1 - buf->rect.x0,
-						    buf->rect.y1 - buf->rect.y0,
-						    buf->buf_rowstride,
-						    NULL, NULL);
-
-
-	    x0 = floor (render_affine[4] - buf->rect.x0 + 0.5);
-	    y0 = floor (render_affine[5] - buf->rect.y0 + 0.5);
-
-	    x1 = x0 + floor (gdk_pixbuf_get_width (priv->pixbuf) * render_affine[0] + 0.5);
-	    y1 = y0 + floor (gdk_pixbuf_get_height (priv->pixbuf) * render_affine[3] + 0.5);
-
-	    x0 = MAX (x0, 0);
-	    x0 = MIN (x0, buf->rect.x1 - buf->rect.x0);
-	    y0 = MAX (y0, 0);
-	    y0 = MIN (y0, buf->rect.y1 - buf->rect.y0);
-	    
-	    x1 = MAX (x1, 0);
-	    x1 = MIN (x1, buf->rect.x1 - buf->rect.x0);
-	    y1 = MAX (y1, 0);
-	    y1 = MIN (y1, buf->rect.y1 - buf->rect.y0);
-	    
-	    gdk_pixbuf_composite  (priv->pixbuf,
-				   dest_pixbuf,
-				   x0, y0,
-				   x1 - x0, y1 - y0,
-				   render_affine[4] - buf->rect.x0,
-				   render_affine[5] - buf->rect.y0,
-				   render_affine[0],
-				   render_affine[3],
-				   GDK_INTERP_NEAREST,
-				   255);
-
-	    gdk_pixbuf_unref (dest_pixbuf);
-	  }
-	else if (gdk_pixbuf_get_has_alpha(priv->pixbuf))
+	if (gdk_pixbuf_get_has_alpha(priv->pixbuf))
 		art_rgb_rgba_affine (buf->buf,
 				     buf->rect.x0, buf->rect.y0, buf->rect.x1, buf->rect.y1,
 				     buf->buf_rowstride,
