@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * $Id: gnome-canvas-text.c,v 1.81 2002/01/15 22:16:06 jody Exp $
+ * $Id: gnome-canvas-text.c,v 1.63 2001/07/04 13:34:31 michael Exp $
  * Copyright (C) 1997, 1998, 1999, 2000 Free Software Foundation
  * All rights reserved.
  *
@@ -376,7 +376,7 @@ gnome_canvas_text_class_init (GnomeCanvasTextClass *class)
                  PROP_ANCHOR,
                  g_param_spec_enum ("anchor", NULL, NULL,
                                     GTK_TYPE_ANCHOR_TYPE,
-                                    GTK_ANCHOR_CENTER,
+                                    GTK_ANCHOR_NW,
                                     (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property
                 (gobject_class,
@@ -531,8 +531,15 @@ gnome_canvas_text_init (GnomeCanvasText *text)
 	text->yofs = 0.0;
 	text->layout = NULL;
 
-	text->font_desc = NULL;
+	text->font_desc.family_name = NULL;
 	
+	text->family_set  = FALSE;
+	text->style_set   = FALSE;
+	text->variant_set = FALSE;
+	text->weight_set  = FALSE;
+	text->stretch_set = FALSE;
+	text->size_set    = FALSE;
+
 	text->underline     = PANGO_UNDERLINE_NONE;
 	text->strikethrough = FALSE;
 	text->rise          = 0;
@@ -541,9 +548,9 @@ gnome_canvas_text_init (GnomeCanvasText *text)
 	text->strike_set    = FALSE;
 	text->rise_set      = FALSE;
 	
-	text->priv = g_new (GnomeCanvasTextPrivate, 1);
-	text->priv->bitmap.buffer = NULL;
-	text->priv->render_dirty = 1;
+	text->private = g_new(GnomeCanvasTextPrivate, 1);
+	text->private->bitmap.buffer = NULL;
+	text->private->render_dirty = 1;
 }
 
 /* Destroy handler for the text item */
@@ -565,10 +572,9 @@ gnome_canvas_text_destroy (GtkObject *object)
 	    g_object_unref (G_OBJECT (text->layout));
 	text->layout = NULL;
 	
-	if (text->font_desc) {
-		pango_font_description_free (text->font_desc);
-		text->font_desc = NULL;
-	}
+	if (text->font_desc.family_name)
+		g_free (text->font_desc.family_name);
+	text->font_desc.family_name = NULL;
 
 	if (text->attr_list)
 		pango_attr_list_unref (text->attr_list);
@@ -578,16 +584,109 @@ gnome_canvas_text_destroy (GtkObject *object)
 		gdk_bitmap_unref (text->stipple);
 	text->stipple = NULL;
 
-	if (text->priv && text->priv->bitmap.buffer) {
-		g_free (text->priv->bitmap.buffer);		
+	if (text->private && text->private->bitmap.buffer) {
+		g_free (text->private->bitmap.buffer);		
 	}
-	g_free (text->priv);
-	text->priv = NULL;
+	g_free (text->private);
+	text->private = NULL;
 	
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
+static void
+get_bounds_item_relative (GnomeCanvasText *text, double *px1, double *py1, double *px2, double *py2)
+{
+	GnomeCanvasItem *item;
+	double x, y;
+	double clip_x, clip_y;
+
+	item = GNOME_CANVAS_ITEM (text);
+
+	x = text->x;
+	y = text->y;
+
+	clip_x = x;
+	clip_y = y;
+
+	/* Calculate text dimensions */
+
+	if (text->layout)
+	        pango_layout_get_pixel_size (text->layout,
+					     &text->max_width,
+					     &text->height);
+	else
+		text->height = 0;
+
+	/* Anchor text */
+
+	switch (text->anchor) {
+	case GTK_ANCHOR_NW:
+	case GTK_ANCHOR_W:
+	case GTK_ANCHOR_SW:
+		break;
+
+	case GTK_ANCHOR_N:
+	case GTK_ANCHOR_CENTER:
+	case GTK_ANCHOR_S:
+		x -= text->max_width / 2;
+		clip_x -= text->clip_width / 2;
+		break;
+
+	case GTK_ANCHOR_NE:
+	case GTK_ANCHOR_E:
+	case GTK_ANCHOR_SE:
+		x -= text->max_width;
+		clip_x -= text->clip_width;
+		break;
+
+	default:
+		break;
+	}
+
+	switch (text->anchor) {
+	case GTK_ANCHOR_NW:
+	case GTK_ANCHOR_N:
+	case GTK_ANCHOR_NE:
+		break;
+
+	case GTK_ANCHOR_W:
+	case GTK_ANCHOR_CENTER:
+	case GTK_ANCHOR_E:
+		y -= text->height / 2;
+		clip_y -= text->clip_height / 2;
+		break;
+
+	case GTK_ANCHOR_SW:
+	case GTK_ANCHOR_S:
+	case GTK_ANCHOR_SE:
+		y -= text->height;
+		clip_y -= text->clip_height;
+		break;
+
+	default:
+		break;
+	}
+
+	/* Bounds */
+
+
+	
+
+	if (text->clip) {
+		/* maybe do bbox intersection here? */
+		*px1 = clip_x;
+		*py1 = clip_y;
+		*px2 = clip_x + text->clip_width;
+		*py2 = clip_y + text->clip_height;
+	} else {
+		*px1 = x;
+		*py1 = y;
+		*px2 = x + text->max_width;
+		*py2 = y + text->height;
+	}
+}
+
 static void
 get_bounds (GnomeCanvasText *text, double *px1, double *py1, double *px2, double *py2)
 {
@@ -610,6 +709,15 @@ get_bounds (GnomeCanvasText *text, double *px1, double *py1, double *px2, double
 	text->clip_cwidth = text->clip_width * item->canvas->pixels_per_unit;
 	text->clip_cheight = text->clip_height * item->canvas->pixels_per_unit;
 
+	/* Calculate text dimensions */
+
+	if (text->layout)
+	        pango_layout_get_pixel_size (text->layout,
+					     &text->max_width,
+					     &text->height);
+	else
+		text->height = 0;
+
 	/* Anchor text */
 
 	switch (text->anchor) {
@@ -675,6 +783,22 @@ get_bounds (GnomeCanvasText *text, double *px1, double *py1, double *px2, double
 	}
 }
 
+/* Recalculates the bounding box of the text item.  The bounding box is defined
+ * by the text's extents if the clip rectangle is disabled.  If it is enabled,
+ * the bounding box is defined by the clip rectangle itself.
+ */
+static void
+recalc_bounds (GnomeCanvasText *text)
+{
+	GnomeCanvasItem *item;
+
+	item = GNOME_CANVAS_ITEM (text);
+
+	get_bounds (text, &item->x1, &item->y1, &item->x2, &item->y2);
+
+	gnome_canvas_group_child_bounds (GNOME_CANVAS_GROUP (item->parent), item);
+}
+
 /* Convenience function to set the text's GC's foreground color */
 static void
 set_text_gc_foreground (GnomeCanvasText *text)
@@ -708,35 +832,6 @@ set_stipple (GnomeCanvasText *text, GdkBitmap *stipple, int reconfigure)
 	}
 }
 
-static PangoFontMask
-get_property_font_set_mask (guint prop_id)
-{
-  switch (prop_id)
-    {
-    case PROP_FAMILY_SET:
-      return PANGO_FONT_MASK_FAMILY;
-    case PROP_STYLE_SET:
-      return PANGO_FONT_MASK_STYLE;
-    case PROP_VARIANT_SET:
-      return PANGO_FONT_MASK_VARIANT;
-    case PROP_WEIGHT_SET:
-      return PANGO_FONT_MASK_WEIGHT;
-    case PROP_STRETCH_SET:
-      return PANGO_FONT_MASK_STRETCH;
-    case PROP_SIZE_SET:
-      return PANGO_FONT_MASK_SIZE;
-    }
-
-  return 0;
-}
-
-static void
-ensure_font (GnomeCanvasText *text)
-{
-	if (!text->font_desc)
-		text->font_desc = pango_font_description_new ();
-}
-
 /* Set_arg handler for the text item */
 static void
 gnome_canvas_text_set_property (GObject            *object,
@@ -763,32 +858,17 @@ gnome_canvas_text_set_property (GObject            *object,
 	
 
 	if (!text->layout) {
-
 	        PangoContext *gtk_context, *context;
 		gtk_context = gtk_widget_get_pango_context (GTK_WIDGET (item->canvas));
+
 		
 	        if (item->canvas->aa)  {
 			PangoLanguage *language;
-			gint	pixels, mm;
-			double	dpi_x;
-			double	dpi_y;
-			
-			pixels = gdk_screen_width ();
-			mm = gdk_screen_width_mm ();
-			dpi_x = (((double) pixels * 25.4) / (double) mm);
-			
-			pixels = gdk_screen_height ();
-			mm = gdk_screen_height_mm ();
-			dpi_y = (((double) pixels * 25.4) / (double) mm);
-			
-		        context = pango_ft2_get_context (dpi_x, dpi_y);
+		        context = pango_ft2_get_context ();
 			language = pango_context_get_language (gtk_context);
 			pango_context_set_language (context, language);
 			pango_context_set_base_dir (context,
 						    pango_context_get_base_dir (gtk_context));
-			pango_context_set_font_description (context,
-							    pango_context_get_font_description (gtk_context));
-			
 		} else
 			context = gtk_context;
 			
@@ -806,99 +886,173 @@ gnome_canvas_text_set_property (GObject            *object,
 
 		text->text = g_value_dup_string (value);
 		pango_layout_set_text (text->layout, text->text, -1);
+		recalc_bounds (text);
 
-		text->priv->render_dirty = 1;
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_MARKUP:
 		gnome_canvas_text_set_markup (text,
 					      g_value_get_string (value));
-		text->priv->render_dirty = 1;
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_X:
 		text->x = g_value_get_double (value);
+		recalc_bounds (text);
 		break;
 
 	case PROP_Y:
 		text->y = g_value_get_double (value);
+		recalc_bounds (text);
 		break;
 
 	case PROP_FONT: {
 		const char *font_name;
-		PangoFontDescription *font_desc;
 
 		font_name = g_value_get_string (value);
-		if (font_name)
-			font_desc = pango_font_description_from_string (font_name);
-		else
-			font_desc = NULL;
-		
-		gnome_canvas_text_set_font_desc (text, font_desc);
-		if (font_desc)
-			pango_font_description_free (font_desc);
+		if (font_name) {
+			PangoFontDescription *font_desc;
 
+			font_desc = pango_font_description_from_string (font_name);
+			gnome_canvas_text_set_font_desc (text, font_desc);
+			pango_font_description_free (font_desc);
+			
+			gnome_canvas_text_apply_font_desc (text);
+			recalc_bounds (text);
+		} else {
+			text->family_set  = FALSE;
+			text->style_set   = FALSE;
+			text->variant_set = FALSE;
+			text->weight_set  = FALSE;
+			text->stretch_set = FALSE;
+			text->size_set    = FALSE;
+
+			gnome_canvas_text_apply_font_desc (text);
+		}
+		text->private->render_dirty = 1;
 		break;
 	}
 
 	case PROP_FONT_DESC:
-		gnome_canvas_text_set_font_desc (text, g_value_peek_pointer (value));
+		gnome_canvas_text_set_font_desc (
+			text, g_value_peek_pointer (value));
+		
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_FAMILY:
-	case PROP_STYLE:
-	case PROP_VARIANT:
-	case PROP_WEIGHT:
-	case PROP_STRETCH:
-	case PROP_SIZE:
-	case PROP_SIZE_POINTS:
-		ensure_font (text);
-
-		switch (param_id) {
-		case PROP_FAMILY:
-			pango_font_description_set_family (text->font_desc,
-							   g_value_get_string (value));
-			break;
-		case PROP_STYLE:
-			pango_font_description_set_style (text->font_desc,
-							  g_value_get_enum (value));
-			break;
-		case PROP_VARIANT:
-			pango_font_description_set_variant (text->font_desc,
-							    g_value_get_enum (value));
-			break;
-		case PROP_WEIGHT:
-			pango_font_description_set_weight (text->font_desc,
-							   g_value_get_int (value));
-			break;
-		case PROP_STRETCH:
-			pango_font_description_set_stretch (text->font_desc,
-							    g_value_get_enum (value));
-			break;
-		case PROP_SIZE:
-			/* FIXME: This is bogus! It should be pixels, not points/PANGO_SCALE! */
-			pango_font_description_set_size (text->font_desc,
-							 g_value_get_int (value));
-			break;
-		case PROP_SIZE_POINTS:
-			pango_font_description_set_size (text->font_desc,
-							 g_value_get_double (value) * PANGO_SCALE);
-			break;
-		}
+		text->font_desc.family_name = g_strdup (g_value_get_string (value));
+		text->family_set = TRUE;
 		
 		gnome_canvas_text_apply_font_desc (text);
-		text->priv->render_dirty = 1;
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_FAMILY_SET:
+		text->family_set = g_value_get_boolean (value);
+
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
+		break;
+		
+	case PROP_STYLE:
+		text->font_desc.style = g_value_get_enum (value);
+		text->style_set = TRUE;
+		
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
+		break;
+
 	case PROP_STYLE_SET:
+		text->style_set = g_value_get_boolean (value);
+
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
+		break;
+		
+	case PROP_VARIANT:
+		text->font_desc.variant = g_value_get_enum (value);
+		text->variant_set = TRUE;
+		
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
+		break;
+
 	case PROP_VARIANT_SET:
+		text->variant_set = g_value_get_boolean (value);
+
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
+		break;
+		
+	case PROP_WEIGHT:
+		text->font_desc.weight = g_value_get_int (value);
+		text->weight_set = TRUE;
+		
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
+		break;
+
 	case PROP_WEIGHT_SET:
+		text->weight_set = g_value_get_boolean (value);
+
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
+		break;
+
+	case PROP_STRETCH:
+		text->font_desc.stretch = g_value_get_enum (value);
+		text->stretch_set = TRUE;
+		
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
+		break;
+
 	case PROP_STRETCH_SET:
+		text->stretch_set = g_value_get_boolean (value);
+
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
+		break;
+
+	case PROP_SIZE:
+		text->font_desc.size = g_value_get_int (value);
+		text->size_set = TRUE;
+		
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
+		break;
+
+	case PROP_SIZE_POINTS:
+		text->font_desc.size = g_value_get_double (value) * PANGO_SCALE;
+		text->size_set = TRUE;
+		
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
+		break;
+
 	case PROP_SIZE_SET:
-		if (!g_value_get_boolean (value) && text->font_desc)
-			pango_font_description_unset_fields (text->font_desc,
-							     get_property_font_set_mask (param_id));
+		text->size_set = g_value_get_boolean (value);
+
+		gnome_canvas_text_apply_font_desc (text);
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_SCALE:
@@ -906,14 +1060,16 @@ gnome_canvas_text_set_property (GObject            *object,
 		text->scale_set = TRUE;
 		
 		gnome_canvas_text_apply_font_desc (text);
-		text->priv->render_dirty = 1;
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 		
 	case PROP_SCALE_SET:
 		text->scale_set = g_value_get_boolean (value);
 		
 		gnome_canvas_text_apply_font_desc (text);
-		text->priv->render_dirty = 1;
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;		
 		
 	case PROP_UNDERLINE:
@@ -921,14 +1077,16 @@ gnome_canvas_text_set_property (GObject            *object,
 		text->underline_set = TRUE;
 		
 		gnome_canvas_text_apply_attributes (text);
-		text->priv->render_dirty = 1;
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_UNDERLINE_SET:
 		text->underline_set = g_value_get_boolean (value);
 		
 		gnome_canvas_text_apply_attributes (text);
-		text->priv->render_dirty = 1;
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_STRIKETHROUGH:
@@ -936,14 +1094,16 @@ gnome_canvas_text_set_property (GObject            *object,
 		text->strike_set = TRUE;
 		
 		gnome_canvas_text_apply_attributes (text);
-		text->priv->render_dirty = 1;
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_STRIKETHROUGH_SET:
 		text->strike_set = g_value_get_boolean (value);
 		
 		gnome_canvas_text_apply_attributes (text);
-		text->priv->render_dirty = 1;
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_RISE:
@@ -951,14 +1111,16 @@ gnome_canvas_text_set_property (GObject            *object,
 		text->rise_set = TRUE;
 		
 		gnome_canvas_text_apply_attributes (text);
-		text->priv->render_dirty = 1;
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_RISE_SET:
 		text->rise_set = TRUE;
 		
 		gnome_canvas_text_apply_attributes (text);
-		text->priv->render_dirty = 1;
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_ATTRIBUTES:
@@ -968,11 +1130,13 @@ gnome_canvas_text_set_property (GObject            *object,
 		text->attr_list = g_value_peek_pointer (value);
 		
 		gnome_canvas_text_apply_attributes (text);
-		text->priv->render_dirty = 1;
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_ANCHOR:
 		text->anchor = g_value_get_enum (value);
+		recalc_bounds (text);
 		break;
 
 	case PROP_JUSTIFICATION:
@@ -994,30 +1158,36 @@ gnome_canvas_text_set_property (GObject            *object,
 			break;
 		}		  
 		pango_layout_set_alignment (text->layout, align);
-		text->priv->render_dirty = 1;				
+		recalc_bounds (text);
+		text->private->render_dirty = 1;				
 		break;
 
 	case PROP_CLIP_WIDTH:
 		text->clip_width = fabs (g_value_get_double (value));
-		text->priv->render_dirty = 1;				
+		recalc_bounds (text);
+		text->private->render_dirty = 1;				
 		break;
 
 	case PROP_CLIP_HEIGHT:
 		text->clip_height = fabs (g_value_get_double (value));
-		text->priv->render_dirty = 1;				
+		recalc_bounds (text);
+		text->private->render_dirty = 1;				
 		break;
 
 	case PROP_CLIP:
 		text->clip = g_value_get_boolean (value);
-		text->priv->render_dirty = 1;
+		recalc_bounds (text);
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_X_OFFSET:
 		text->xofs = g_value_get_double (value);
+		recalc_bounds (text);
 		break;
 
 	case PROP_Y_OFFSET:
 		text->yofs = g_value_get_double (value);
+		recalc_bounds (text);
 		break;
 
         case PROP_FILL_COLOR: {
@@ -1033,7 +1203,7 @@ gnome_canvas_text_set_property (GObject            *object,
 				      0xff);
 			color_changed = TRUE;
 		}
-		text->priv->render_dirty = 1;
+		text->private->render_dirty = 1;
 		break;
 	}
 
@@ -1058,7 +1228,7 @@ gnome_canvas_text_set_property (GObject            *object,
         case PROP_FILL_COLOR_RGBA:
 		text->rgba = g_value_get_uint (value);
 		color_changed = TRUE;
-		text->priv->render_dirty = 1;
+		text->private->render_dirty = 1;
 		break;
 
 	case PROP_FILL_STIPPLE:
@@ -1078,20 +1248,9 @@ gnome_canvas_text_set_property (GObject            *object,
 
 		if (!item->canvas->aa)
 			set_text_gc_foreground (text);
-	}
 
-	/* Calculate text dimensions */
-
-	if (text->layout)
-	        pango_layout_get_pixel_size (text->layout,
-					     &text->max_width,
-					     &text->height);
-	else {
-		text->max_width = 0;
-		text->height = 0;
+		gnome_canvas_item_request_update (item);
 	}
-	
-	gnome_canvas_item_request_update (item);
 }
 
 /* Get_arg handler for the text item */
@@ -1102,6 +1261,7 @@ gnome_canvas_text_get_property (GObject            *object,
 				GParamSpec         *pspec)
 {
 	GnomeCanvasText *text;
+	GdkColor *color;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GNOME_IS_CANVAS_TEXT (object));
@@ -1121,80 +1281,55 @@ gnome_canvas_text_get_property (GObject            *object,
 		g_value_set_double (value, text->y);
 		break;
 
-	case PROP_FONT:
 	case PROP_FONT_DESC:
+		g_value_set_boxed (value, &(text->font_desc));
+		break;
+
 	case PROP_FAMILY:
-	case PROP_STYLE:
-	case PROP_VARIANT:
-	case PROP_WEIGHT:
-	case PROP_STRETCH:
-	case PROP_SIZE:
-	case PROP_SIZE_POINTS:
-		ensure_font (text);
-		
-		switch (param_id) {
-		case PROP_FONT:
-		{
-			/* FIXME GValue imposes a totally gratuitous string copy
-			 * here, we could just hand off string ownership
-			 */
-			gchar *str;
-			
-			str = pango_font_description_to_string (text->font_desc);
-			g_value_set_string (value, str);
-			g_free (str);
-
-			break;
-		}
-			
-		case PROP_FONT_DESC:
-			g_value_set_boxed (value, text->font_desc);
-			break;
-
-		case PROP_FAMILY:
-			g_value_set_string (value, pango_font_description_get_family (text->font_desc));
-			break;
-			
-		case PROP_STYLE:
-			g_value_set_enum (value, pango_font_description_get_style (text->font_desc));
-			break;
-			
-		case PROP_VARIANT:
-			g_value_set_enum (value, pango_font_description_get_variant (text->font_desc));
-			break;
-			
-		case PROP_WEIGHT:
-			g_value_set_int (value, pango_font_description_get_weight (text->font_desc));
-			break;
-			
-		case PROP_STRETCH:
-			g_value_set_enum (value, pango_font_description_get_stretch (text->font_desc));
-			break;
-			
-		case PROP_SIZE:
-			g_value_set_int (value, pango_font_description_get_size (text->font_desc));
-			break;
-			
-		case PROP_SIZE_POINTS:
-			g_value_set_double (value, ((double)pango_font_description_get_size (text->font_desc)) / (double)PANGO_SCALE);
-			break;
-		}
+		g_value_set_string (value, text->font_desc.family_name);
 		break;
-
 	case PROP_FAMILY_SET:
+		g_value_set_boolean (value, text->family_set);
+		break;
+		
+	case PROP_STYLE:
+		g_value_set_enum (value, text->font_desc.style);
+		break;
 	case PROP_STYLE_SET:
+		g_value_set_boolean (value, text->style_set);
+		break;
+		
+	case PROP_VARIANT:
+		g_value_set_enum (value, text->font_desc.variant);
+		break;
 	case PROP_VARIANT_SET:
-	case PROP_WEIGHT_SET:
-	case PROP_STRETCH_SET:
-	case PROP_SIZE_SET:
-	{
-		PangoFontMask set_mask = text->font_desc ? pango_font_description_get_set_fields (text->font_desc) : 0;
-		PangoFontMask test_mask = get_property_font_set_mask (param_id);
-		g_value_set_boolean (value, (set_mask & test_mask) != 0);
-
+		g_value_set_boolean (value, text->variant_set);
 		break;
-	}
 
+	case PROP_WEIGHT:
+		g_value_set_int (value, text->font_desc.weight);
+		break;
+	case PROP_WEIGHT_SET:
+		g_value_set_boolean (value, text->weight_set);
+		break;
+		
+	case PROP_STRETCH:
+		g_value_set_enum (value, text->font_desc.stretch);
+		break;
+	case PROP_STRETCH_SET:
+		g_value_set_boolean (value, text->stretch_set);
+		break;
+		
+	case PROP_SIZE:
+		g_value_set_int (value, text->font_desc.size);
+		break;
+	case PROP_SIZE_POINTS:
+		g_value_set_double (value, ((double)text->font_desc.size) / (double)PANGO_SCALE);
+		break;
+	case PROP_SIZE_SET:
+		g_value_set_boolean (value, text->size_set);
+		break;
+		
 	case PROP_SCALE:
 		g_value_set_double (value, text->scale);
 		break;
@@ -1236,7 +1371,7 @@ gnome_canvas_text_get_property (GObject            *object,
 		break;
 
 	case PROP_CLIP_WIDTH:
-		g_value_set_double (value, text->clip_width);
+		g_value_set_enum (value, text->clip_width);
 		break;
 
 	case PROP_CLIP_HEIGHT:
@@ -1255,21 +1390,14 @@ gnome_canvas_text_get_property (GObject            *object,
 		g_value_set_double (value, text->yofs);
 		break;
 
-	case PROP_FILL_COLOR:
-                g_value_set_string_take_ownership (value,
-						   g_strdup_printf ("#%02x%02x%02x",
-                                                                    text->rgba >> 24,
-                                                                    (text->rgba >> 16) & 0xff,
-                                                                    (text->rgba >> 8) & 0xff));
-		break;
-
 	case PROP_FILL_COLOR_GDK: {
-		GnomeCanvas *canvas = GNOME_CANVAS_ITEM (text)->canvas;
-		GdkColormap *colormap = gtk_widget_get_colormap (GTK_WIDGET (canvas));
-		GdkColor color;
+		GdkColormap *colormap;
 
-		gdk_colormap_query_color (colormap, text->pixel, &color);
-		g_value_set_boxed (value, &color);
+		color = g_new (GdkColor, 1);
+		color->pixel = text->pixel;
+		colormap = gtk_widget_get_colormap (GTK_WIDGET (text->item.canvas));
+		gdk_rgb_find_color (colormap, color);
+		g_value_set_boxed (value, color);
 		break;
 	}
 	case PROP_FILL_COLOR_RGBA:
@@ -1277,7 +1405,7 @@ gnome_canvas_text_get_property (GObject            *object,
 		break;
 
 	case PROP_FILL_STIPPLE:
-		g_value_set_object (value, text->stipple);
+		g_value_set_boxed (value, text->stipple);
 		break;
 
 	case PROP_TEXT_WIDTH:
@@ -1301,11 +1429,34 @@ gnome_canvas_text_apply_font_desc (GnomeCanvasText *text)
 	PangoFontDescription *font_desc =
 		pango_font_description_copy (
 			GTK_WIDGET (GNOME_CANVAS_ITEM (text)->canvas)->style->font_desc);
+	
+	if (text->family_set && text->font_desc.family_name)
+	{
+		if (font_desc->family_name)
+			g_free (font_desc->family_name);
+		font_desc->family_name = g_strdup (text->font_desc.family_name);
+	}
+		
+	if (text->style_set)
+		font_desc->style = text->font_desc.style;
 
-	if (text->font_desc)
-		pango_font_description_merge (font_desc, text->font_desc, TRUE);
+	if (text->variant_set)
+		font_desc->variant = text->font_desc.variant;
 
-	pango_layout_set_font_description (text->layout, font_desc);
+	if (text->weight_set)
+		font_desc->weight = text->font_desc.weight;
+	
+	if (text->stretch_set)
+		font_desc->stretch = text->font_desc.stretch;
+
+	if (text->size_set)
+		font_desc->size = text->font_desc.size;
+
+	if (text->scale_set)
+		font_desc->size *= text->scale;
+	
+	pango_layout_set_font_description (text->layout,
+					   font_desc);
 	pango_font_description_free (font_desc);
 }
 
@@ -1345,16 +1496,23 @@ static void
 gnome_canvas_text_set_font_desc (GnomeCanvasText      *text,
 				 PangoFontDescription *font_desc)
 {
-	if (text->font_desc)
-		pango_font_description_free (text->font_desc);
+	if (text->font_desc.family_name)
+		g_free (text->font_desc.family_name);
 
-	if (font_desc)
-		text->font_desc = pango_font_description_copy (font_desc);
-	else
-		text->font_desc = NULL;
+	text->font_desc.family_name = g_strdup (font_desc->family_name);
 
-	gnome_canvas_text_apply_font_desc (text);
-	text->priv->render_dirty = 1;
+	text->font_desc.style   = font_desc->style;
+	text->font_desc.variant = font_desc->variant;
+	text->font_desc.weight  = font_desc->weight;
+	text->font_desc.stretch = font_desc->stretch;
+	text->font_desc.size    = font_desc->size;
+	
+	text->family_set  = TRUE;
+	text->style_set   = TRUE;
+	text->variant_set = TRUE;
+	text->weight_set  = TRUE;
+	text->stretch_set = TRUE;
+	text->size_set    = TRUE;
 }
 
 /* Setting the text from a Pango markup string */
@@ -1396,19 +1554,31 @@ gnome_canvas_text_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_pa
 {
 	GnomeCanvasText *text;
 	double x1, y1, x2, y2;
+	ArtDRect i_bbox, c_bbox;
+	int i;
 
 	text = GNOME_CANVAS_TEXT (item);
 
 	if (parent_class->update)
 		(* parent_class->update) (item, affine, clip_path, flags);
 
-	set_text_gc_foreground (text);
-	set_stipple (text, text->stipple, TRUE);
-	get_bounds (text, &x1, &y1, &x2, &y2);
+	if (!item->canvas->aa) {
+		set_text_gc_foreground (text);
+		set_stipple (text, text->stipple, TRUE);
+		get_bounds (text, &x1, &y1, &x2, &y2);
 
-	gnome_canvas_update_bbox (item,
-				  floor (x1), floor (y1),
-				  ceil (x2), ceil (y2));
+		gnome_canvas_update_bbox (item, x1, y1, x2, y2);
+	} else {
+		/* aa rendering */
+		for (i = 0; i < 6; i++)
+			text->affine[i] = affine[i];
+		get_bounds_item_relative (text, &i_bbox.x0, &i_bbox.y0, &i_bbox.x1, &i_bbox.y1);
+		art_drect_affine_transform (&c_bbox, &i_bbox, affine);
+
+		
+		gnome_canvas_update_bbox (item, floor(c_bbox.x0), floor(c_bbox.y0), ceil(c_bbox.x1), ceil(c_bbox.y1));
+
+	}
 }
 
 /* Realize handler for the text item */
@@ -1479,38 +1649,36 @@ gnome_canvas_text_render (GnomeCanvasItem *item, GnomeCanvasBuf *buf)
 {
 	GnomeCanvasText *text;
 	guint32 fg_color;
+	double affine[6], anchor_affine[6], anchor_x = 0.0, anchor_y = 0.0,
+		bitmap_width, bitmap_height;
+
+
 	int render_x = 0, render_y = 0; /* offsets for text rendering,
 					 * for clipping rectangles */
-	int x, y;
-	int w, h;
-	guchar *dst, *src;
-	int src_dx, src_dy;
-	int i, alpha;
-	int bm_rows, bm_width;
+
+
 	
 	text = GNOME_CANVAS_TEXT (item);
 
 	if (!text->text)
 		return;
 
-	fg_color = text->rgba;
+	fg_color = text->rgba >> 8;
 
         gnome_canvas_buf_ensure_buf (buf);
 
-	bm_rows = (text->clip) ? text->clip_cheight : text->height;
-	bm_width = (text->clip) ? text->clip_cwidth : text->max_width;
-	if(text->priv->render_dirty ||
-	   bm_rows != text->priv->bitmap.rows ||
-	   bm_width != text->priv->bitmap.width) {		
-		if(text->priv->bitmap.buffer) {
-			g_free(text->priv->bitmap.buffer);
+	
+	if(text->private->render_dirty) {		
+		if(text->private->bitmap.buffer) {
+			g_free(text->private->bitmap.buffer);
 		}
-		text->priv->bitmap.rows =  bm_rows;
-		text->priv->bitmap.width = bm_width;
-		text->priv->bitmap.pitch = (text->priv->bitmap.width+3)&~3;
-		text->priv->bitmap.buffer = g_malloc0 (text->priv->bitmap.rows * text->priv->bitmap.pitch);
-		text->priv->bitmap.num_grays = 256;
-		text->priv->bitmap.pixel_mode = ft_pixel_mode_grays;
+		text->private->bitmap.rows =  (text->clip) ? text->clip_height : text->height;
+		text->private->bitmap.width = (text->clip) ? text->clip_width : text->max_width;
+		text->private->bitmap.pitch = (text->max_width+3)&~3;
+		text->private->bitmap.buffer = g_malloc0 (text->private->bitmap.rows * text->private->bitmap.pitch);
+		text->private->bitmap.num_grays = 256;
+		text->private->bitmap.pixel_mode = ft_pixel_mode_grays;
+
 
 		/* What this does is when a clipping rectangle is
 		   being used shift the rendering of the text by the
@@ -1522,68 +1690,114 @@ gnome_canvas_text_render (GnomeCanvasItem *item, GnomeCanvasBuf *buf)
 		   rasterizing it. */
 
 		if(text->clip) {
-			render_x = text->cx - text->clip_cx;
-			render_y = text->cy - text->clip_cy;
+			switch(text->anchor) {
+			case GTK_ANCHOR_NW:				
+				break;
+			case GTK_ANCHOR_W:
+				render_y -= (text->height-text->clip_height)*0.5;
+				break;
+			case GTK_ANCHOR_SW:
+				/* you don't want this */
+				render_y -= (text->height-text->clip_height);
+				break;
+			case GTK_ANCHOR_N:
+				render_x -= (text->max_width-text->clip_width)*0.5;
+				break;
+			case GTK_ANCHOR_CENTER:
+				render_x -= (text->max_width-text->clip_width)*0.5;
+				render_y -= (text->height-text->clip_height)*0.5;
+				break;
+			case GTK_ANCHOR_S:
+				render_x -= (text->max_width-text->clip_width)*0.5;
+				render_y -= (text->height-text->clip_height);
+				break;
+			case GTK_ANCHOR_NE:
+				render_x -= (text->max_width-text->clip_width);
+				break;
+			case GTK_ANCHOR_E:
+				render_x -= (text->max_width-text->clip_width);
+				render_y -= (text->height-text->clip_height)*0.5;
+				break;
+			case GTK_ANCHOR_SE:
+			        render_x -= (text->max_width-text->clip_width);
+				render_y -= (text->height-text->clip_height);
+			default:
+				break;			
+			}
 		}
-
-		pango_ft2_render_layout (&text->priv->bitmap, text->layout, render_x, render_y);
-
-		text->priv->render_dirty = 0;
-	}
-
-	if (text->clip) {
-		x = text->clip_cx - buf->rect.x0;
-		y = text->clip_cy - buf->rect.y0;
-	} else {
-		x = text->cx - buf->rect.x0;
-		y = text->cy - buf->rect.y0;
-	}
 		
-	w = text->priv->bitmap.width;
-	h = text->priv->bitmap.rows;
 
-	src_dx = src_dy = 0;
-	
-	if (x + w > buf->rect.x1 - buf->rect.x0) {
-		w = buf->rect.x1 - buf->rect.x0 - x;
-	}
-	
-	if (y + h > buf->rect.y1 - buf->rect.y0) {
-		h = buf->rect.y1 - buf->rect.y0 - y;
+		pango_ft2_render_layout (&text->private->bitmap, text->layout, render_x, render_y);
+		
+		text->private->render_dirty = 0;
 	}
 
-	if (x < 0) {
-		w -= - x;
-		src_dx += - x;
-		x = 0;
-	}
+
+
+	memcpy(affine, text->affine, sizeof(gdouble)*6);       
+
+	anchor_x = text->x*item->canvas->pixels_per_unit;
+	anchor_y = text->y*item->canvas->pixels_per_unit;
+
+	bitmap_width = text->private->bitmap.width;
+	bitmap_height = text->private->bitmap.rows;
+
 	
-	if (y < 0) {
-		h -= -y;
-		src_dy += - y;
-		y = 0;
-	}
-	
-	dst = buf->buf + y * buf->buf_rowstride + x * 3;
-	src = text->priv->bitmap.buffer +
-		src_dy * text->priv->bitmap.pitch + src_dx;
-	while (h-- > 0) {
-		i = w;
-		while (i-- > 0) {
-			/* FIXME: Do the libart thing instead of divide by 255 */
-			alpha = ((fg_color & 0xff) * (*src)) / 255;
-			dst[0] = (dst[0] * (255 - alpha) + ((fg_color >> 24) & 0xff) * alpha) / 255;
-			dst[1] = (dst[1] * (255 - alpha) + ((fg_color >> 16) & 0xff) * alpha) / 255;
-			dst[2] = (dst[2] * (255 - alpha) + ((fg_color >> 8) & 0xff) * alpha) / 255;
-			dst += 3;
-			src += 1;
-		}
-		dst += buf->buf_rowstride - w*3;
-		src += text->priv->bitmap.pitch - w;
+	/* Figure out the correct translation to apply to affine due
+	 * to the anchor */
+	switch(text->anchor) {
+	case GTK_ANCHOR_NW:
+		break;
+	case GTK_ANCHOR_W:
+		anchor_y -= (bitmap_height*0.5)*item->canvas->pixels_per_unit;
+		break;
+	case GTK_ANCHOR_SW:
+		anchor_y -= bitmap_height*item->canvas->pixels_per_unit;
+		break;
+	case GTK_ANCHOR_N:
+		anchor_x -= (bitmap_width*0.5)*item->canvas->pixels_per_unit;
+		break;
+	case GTK_ANCHOR_CENTER:
+		anchor_x -= (bitmap_width*0.5)*item->canvas->pixels_per_unit;
+		anchor_y -= (bitmap_height*0.5)*item->canvas->pixels_per_unit;
+		break;
+	case GTK_ANCHOR_S:
+		anchor_x -= (bitmap_width*0.5)*item->canvas->pixels_per_unit;
+		anchor_y -= bitmap_height*item->canvas->pixels_per_unit;
+		break;
+	case GTK_ANCHOR_NE:
+		anchor_x -= (bitmap_width)*item->canvas->pixels_per_unit;
+		break;
+	case GTK_ANCHOR_E:
+		anchor_x -= bitmap_width*item->canvas->pixels_per_unit;
+		anchor_y -= (bitmap_height*0.5)*item->canvas->pixels_per_unit;
+		break;
+	case GTK_ANCHOR_SE:
+		anchor_x -= bitmap_width*item->canvas->pixels_per_unit;
+		anchor_y -= bitmap_height*item->canvas->pixels_per_unit;
+	default:
+		break;
 	}
+
+	/* Honor the anchor offsets */
+	anchor_x += text->xofs;
+	anchor_y += text->yofs;
 	
+	art_affine_translate(anchor_affine, anchor_x, anchor_y);
+	art_affine_multiply(affine, affine, anchor_affine);
+
+	art_rgb_a_affine (buf->buf,
+			  buf->rect.x0, buf->rect.y0, buf->rect.x1, buf->rect.y1,
+			  buf->buf_rowstride,
+			  text->private->bitmap.buffer,
+			  text->private->bitmap.width,
+			  text->private->bitmap.rows,
+			  text->private->bitmap.pitch,
+			  fg_color,
+			  affine,
+			  ART_FILTER_NEAREST, NULL);
+			
 	buf->is_bg = 0;
-	return;
 }
 
 /* Point handler for the text item */
@@ -1619,7 +1833,7 @@ gnome_canvas_text_point (GnomeCanvasItem *item, double x, double y,
 			x1 = PANGO_PIXELS (log_rect.x);
 			y1 = PANGO_PIXELS (log_rect.y);
 			x2 = PANGO_PIXELS (log_rect.x+log_rect.width);
-			y2 = PANGO_PIXELS (log_rect.y+log_rect.height);
+			y2 = PANGO_PIXELS (log_rect.x+log_rect.height);
 
 
 			if (x1 < text->clip_cx)
@@ -1659,10 +1873,8 @@ gnome_canvas_text_point (GnomeCanvasItem *item, double x, double y,
 		else
 			dy = 0;
 
-		if ((dx == 0) && (dy == 0)) {
-			pango_layout_iter_free(iter);
+		if ((dx == 0) && (dy == 0))
 			return 0.0;
-		}
 
 		dist = sqrt (dx * dx + dy * dy);
 		if (dist < best)
@@ -1670,8 +1882,6 @@ gnome_canvas_text_point (GnomeCanvasItem *item, double x, double y,
 		
 	} while (pango_layout_iter_next_line(iter));
 
-	pango_layout_iter_free(iter);
-	
 	return best / item->canvas->pixels_per_unit;
 }
 
