@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: 8; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 1997, 1998, 1999, 2000 Free Software Foundation
  * All rights reserved.
@@ -45,6 +44,9 @@
 #include <libart_lgpl/art_uta.h>
 #include <libart_lgpl/art_affine.h>
 
+/* EEK!, gdk leaks some Xlib stuff, such as the grab success codes */
+#include <X11/X.h>
+
 G_BEGIN_DECLS
 
 
@@ -198,7 +200,7 @@ struct _GnomeCanvasItemClass {
 
 	/* Render the item over the buffer given.  The buf data structure
 	 * contains both a pointer to a packed 24-bit RGB array, and the
-	 * coordinates.  This method is only used for antialiased canvases.
+	 * coordinates.  This method is only used for libart-based canvases.
 	 *
 	 * TODO: figure out where clip paths fit into the rendering framework.
 	 */
@@ -222,19 +224,24 @@ struct _GnomeCanvasItemClass {
 	 * coordinates are in the canvas world coordinate system.
 	 */
 	gboolean (* event)                (GnomeCanvasItem *item, GdkEvent *event);
-
-	/* Reserved for future expansion */
-	gpointer spare_vmethods [4];
+        gboolean (* button_press_event)   (GnomeCanvasItem *item, GdkEventButton *event);
+        gboolean (* button_release_event) (GnomeCanvasItem *item, GdkEventButton *event);
+        gboolean (* motion_notify_event)  (GnomeCanvasItem *item, GdkEventMotion *event);
+        gboolean (* key_press_event)      (GnomeCanvasItem *item, GdkEventKey *event);
+        gboolean (* key_release_event)    (GnomeCanvasItem *item, GdkEventKey *event);
+        gboolean (* enter_notify_event)   (GnomeCanvasItem *item, GdkEventCrossing *event);
+        gboolean (* leave_notify_event)   (GnomeCanvasItem *item, GdkEventCrossing *event);
 };
 
 
-GType gnome_canvas_item_get_type (void) G_GNUC_CONST;
+/* Standard Gtk function */
+GtkType gnome_canvas_item_get_type (void) G_GNUC_CONST;
 
 /* Create a canvas item using the standard Gtk argument mechanism.  The item is
  * automatically inserted at the top of the specified canvas group.  The last
  * argument must be a NULL pointer.
  */
-GnomeCanvasItem *gnome_canvas_item_new (GnomeCanvasGroup *parent, GType type,
+GnomeCanvasItem *gnome_canvas_item_new (GnomeCanvasGroup *parent, GtkType type,
 					const gchar *first_arg_name, ...);
 
 /* Constructors for use in derived classes and language wrappers */
@@ -259,6 +266,14 @@ void gnome_canvas_item_affine_relative (GnomeCanvasItem *item, const double affi
 /* Apply an absolute affine transformation to the item. */
 void gnome_canvas_item_affine_absolute (GnomeCanvasItem *item, const double affine[6]);
 
+/* Scale an item about a point by the specified factors */
+void gnome_canvas_item_scale (GnomeCanvasItem *item,
+			      double x, double y,
+			      double scale_x, double scale_y);
+
+/* Rotate an item about a point by the specified number of degrees */
+void gnome_canvas_item_rotate (GnomeCanvasItem *item, double x, double y, double angle);
+
 /* Raise an item in the z-order of its parent group by the specified number of
  * positions.
  */
@@ -378,6 +393,11 @@ struct _GnomeCanvasGroup {
 	/* Children of the group */
 	GList *item_list;
 	GList *item_list_end;
+
+	/* The position of the group has been subsumed into the xform of all items */
+#ifdef OLD_XFORM
+	double xpos, ypos;	/* Point that defines the group's origin */
+#endif
 };
 
 struct _GnomeCanvasGroupClass {
@@ -385,7 +405,11 @@ struct _GnomeCanvasGroupClass {
 };
 
 
-GType gnome_canvas_group_get_type (void) G_GNUC_CONST;
+/* Standard Gtk function */
+GtkType gnome_canvas_group_get_type (void) G_GNUC_CONST;
+
+/* Deprecated.  FIXME: remove this */
+void gnome_canvas_group_child_bounds (GnomeCanvasGroup *group, GnomeCanvasItem *item);
 
 
 /*** GnomeCanvas ***/
@@ -460,11 +484,6 @@ struct _GnomeCanvas {
 	/* Tolerance distance for picking items */
 	int close_enough;
 
-	/* Whether the canvas should center the scroll region in the middle of
-	 * the window if the scroll region is smaller than the window.
-	 */
-	unsigned int center_scroll_region : 1;
-
 	/* Whether items need update at next idle loop iteration */
 	unsigned int need_update : 1;
 
@@ -482,35 +501,21 @@ struct _GnomeCanvas {
 
 	/* Whether the canvas is in antialiased mode or not */
 	unsigned int aa : 1;
-
-	/* Which dither mode to use for antialiased mode drawing */
-	GdkRgbDither dither;
 };
 
 struct _GnomeCanvasClass {
 	GtkLayoutClass parent_class;
 
-	/* Draw the background for the area given. This method is only used
-	 * for non-antialiased canvases.
-	 */
-	void (* draw_background) (GnomeCanvas *canvas, GdkDrawable *drawable,
-				  int x, int y, int width, int height);
-
-	/* Render the background for the buffer given. The buf data structure
-	 * contains both a pointer to a packed 24-bit RGB array, and the
-	 * coordinates. This method is only used for antialiased canvases.
-	 */
-	void (* render_background) (GnomeCanvas *canvas, GnomeCanvasBuf *buf);
-
 	/* Private Virtual methods for groping the canvas inside bonobo */
-	void (* request_update) (GnomeCanvas *canvas);
+	void (* request_update ) (GnomeCanvas *canvas);
 
 	/* Reserved for future expansion */
 	gpointer spare_vmethods [4];
 };
 
 
-GType gnome_canvas_get_type (void) G_GNUC_CONST;
+/* Standard Gtk function */
+GtkType gnome_canvas_get_type (void) G_GNUC_CONST;
 
 /* Creates a new canvas.  You should check that the canvas is created with the
  * proper visual and colormap.  Any visual will do unless you intend to insert
@@ -539,12 +544,6 @@ void gnome_canvas_set_scroll_region (GnomeCanvas *canvas,
 void gnome_canvas_get_scroll_region (GnomeCanvas *canvas,
 				     double *x1, double *y1, double *x2, double *y2);
 
-/* Whether the canvas centers the scroll region if it is smaller than the window */
-void gnome_canvas_set_center_scroll_region (GnomeCanvas *canvas, gboolean center_scroll_region);
-
-/* Returns whether the canvas is set to center the scroll region if it is smaller than the window */
-gboolean gnome_canvas_get_center_scroll_region (GnomeCanvas *canvas);
-
 /* Sets the number of pixels that correspond to one unit in world coordinates */
 void gnome_canvas_set_pixels_per_unit (GnomeCanvas *canvas, double n);
 
@@ -620,15 +619,6 @@ gulong gnome_canvas_get_color_pixel (GnomeCanvas *canvas,
  */
 void gnome_canvas_set_stipple_origin (GnomeCanvas *canvas, GdkGC *gc);
 
-/* Controls the dithering used when the canvas renders.
- * Only applicable to antialiased canvases - ignored by non-antialiased canvases.
- */
-void gnome_canvas_set_dither (GnomeCanvas *canvas, GdkRgbDither dither);
-
-/* Returns the dither mode of an antialiased canvas.
- * Only applicable to antialiased canvases - ignored by non-antialiased canvases.
- */
-GdkRgbDither gnome_canvas_get_dither (GnomeCanvas *canvas);
 
 G_END_DECLS
 
